@@ -3,10 +3,12 @@ package Data::Login;
 use strict;
 use warnings;
 
+use DateTime;
+use Error::Pure qw(err);
 use Mo qw(build default is);
 use Mo::utils 0.21 qw(check_array_object check_isa check_length check_number check_required);
 
-our $VERSION = 0.02;
+our $VERSION = 0.03;
 
 has hash_type => (
 	is => 'ro',
@@ -29,6 +31,14 @@ has roles => (
 	is => 'ro',
 );
 
+has valid_from => (
+	is => 'ro',
+);
+
+has valid_to => (
+	is => 'ro',
+);
+
 sub BUILD {
 	my $self = shift;
 
@@ -50,6 +60,21 @@ sub BUILD {
 	# Check roles.
 	check_array_object($self, 'roles', 'Data::Login::Role', 'Roles');
 
+	# Check valid_from.
+	check_required($self, 'valid_from');
+	check_isa($self, 'valid_from', 'DateTime');
+
+	# Check valid_to.
+	check_isa($self, 'valid_to', 'DateTime');
+	if (defined $self->{'valid_to'}
+		&& DateTime->compare($self->{'valid_from'}, $self->{'valid_to'}) != -1) {
+
+		err "Parameter 'valid_to' must be older than 'valid_from' parameter.",
+			'Value', $self->{'valid_to'},
+			'Valid from', $self->{'valid_from'},
+		;
+	}
+
 	return;
 }
 
@@ -75,6 +100,18 @@ Data::Login - Data object for login.
  my $login_name = $obj->login_name;
  my $password_hash = $obj->password_hash;
  my $roles_ar = $obj->roles;
+ my $valid_from = $obj->valid_from;
+ my $valid_to = $obj->valid_to;
+
+=head1 DESCRIPTION
+
+The intention of this module is to store information about the user logins.
+User logins are active only within a certain time range, and we need a mechanism to
+transition to others.
+
+A real-world example is a database table that follows the same format as this data object,
+with multiple records being valid at different times, e.g. for transfering of Digest from
+obsolete version to new. Or planning of access to system from concrete date.
 
 =head1 METHODS
 
@@ -119,6 +156,18 @@ Possible value is reference to array with L<Data::Login::Role> objects.
 Parameter is optional.
 Default value is [].
 
+=item * C<valid_from>
+
+Date and time of start of use.
+Must be a L<DateTime> object.
+It's required.
+
+=item * C<valid_to>
+
+Date and time of end of use. An undefined value means it is in use.
+Must be a L<DateTime> object.
+It's optional.
+
 =back
 
 Returns instance of object.
@@ -163,6 +212,22 @@ Get roles.
 
 Returns reference to array with L<Data::Login::Role> objects.
 
+=head2 C<valid_from>
+
+ my $valid_from = $obj->valid_from;
+
+Get date and time of start of use.
+
+Returns L<DateTime> object.
+
+=head2 C<valid_to>
+
+ my $valid_to = $obj->valid_to;
+
+Get date and time of end of use.
+
+Returns L<DateTime> object or undef.
+
 =head1 ERRORS
 
  new():
@@ -181,6 +246,16 @@ Returns reference to array with L<Data::Login::Role> objects.
          Parameter 'roles' must be a array.
                  Value: %s
                  Reference: %s
+         Parameter 'valid_from' is required.
+         Parameter 'valid_from' must be a 'DateTime' object.
+                 Value: %s
+                 Reference: %s
+         Parameter 'valid_to' must be a 'DateTime' object.
+                 Value: %s
+                 Reference: %s
+         Parameter 'valid_to' must be older than 'valid_from' parameter.
+                 Value: %s
+                 Valid from: %s
          Roles isn't 'Data::Login::Role' object.
                  Value: %s
                  Reference: %s
@@ -195,12 +270,17 @@ Returns reference to array with L<Data::Login::Role> objects.
  use Data::HashType;
  use Data::Login;
  use Data::Login::Role;
+ use DateTime;
 
  my $obj = Data::Login->new(
          'hash_type' => Data::HashType->new(
-                 'active' => 1,
                  'id' => 1,
                  'name' => 'SHA-512',
+                 'valid_from' => DateTime->new(
+                         'day' => 1,
+                         'month' => 1,
+                         'year' => 2024,
+                 ),
          ),
          'id' => 2,
          'login_name' => 'michal.josef.spacek',
@@ -222,6 +302,11 @@ Returns reference to array with L<Data::Login::Role> objects.
                          'role' => 'Bad',
                  ),
          ],
+         'valid_from' => DateTime->new(
+                 'day' => 1,
+                 'month' => 1,
+                 'year' => 2024,
+         ),
  );
 
  # Print out.
@@ -232,6 +317,7 @@ Returns reference to array with L<Data::Login::Role> objects.
  print "Active roles:\n";
  print join "\n", map { $_->active ? ' - '.$_->role : () } @{$obj->roles};
  print "\n";
+ print 'Valid from: '.$obj->valid_from->ymd."\n";
 
  # Output:
  # Hash type: SHA-512
@@ -241,9 +327,12 @@ Returns reference to array with L<Data::Login::Role> objects.
  # Active roles:
  #  - Admin
  #  - User
+ # Valid from: 2024-01-01
 
 =head1 DEPENDENCIES
 
+L<DateTime>,
+L<Error::Pure>,
 L<Mo>,
 L<Mo::utils>.
 
@@ -265,6 +354,6 @@ BSD 2-Clause License
 
 =head1 VERSION
 
-0.02
+0.03
 
 =cut
